#include <sys/types.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#define CHUNK_SIZE 256*512

FILE *audio_file;
FILE *video_file;
int verbose = 0;

void parse_chunk(unsigned char buf[CHUNK_SIZE])
{
  int num_recs = buf[0];
  int i;
  unsigned char *p;
  int total_audio = 0;
  int total_video = 0;
  int ofs = 0;
  int unknown_recs = 0;
  long aposn;
  unsigned int audsize;

  /*
   * each 128k chunk starts with N 16 byte records that tell you what sorts
   * of things are in the chunk
   */
  p = &buf[4];
  ofs = 4 + num_recs * 16;

  for (i = 0; i < num_recs; i++) {
    unsigned size = (p[0] << 8 | p[1]) << 4 | (p[2] >> 4);
    unsigned type = p[3];

    if (type < 0x10) { p += 16; continue; }

    if ((type == 0xe0)||(type == 0xf0)) {
      if (size != 0x10) {
	fwrite(&buf[ofs], 1, size, video_file);
	if (verbose) { printf("video at rec %d ofs 0x%x size 0x%x\n",
							i, ofs, size); }
      }
      total_video += size;
      ofs += size;
      p += 16;
      continue;
    }
    if (type == 0xc0) {
      audsize= size;
      if ((size != 0x10) || ( (size == 0x10) && (buf[ofs]!=0x00))) {

        if (size==867) { audsize= 864; printf("Ha hit a size 0x867 A rec\n"); }
	fwrite(&buf[ofs], 1, audsize, audio_file);
	if (verbose) { printf("audio at rec %d ofs 0x%x size 0x%x\n",
							i, ofs, size); }
      }
      if ((size == 0x10) && (buf[ofs]!=0x00)) {
	aposn=ftell(audio_file);
	printf("Ha just hit a size 10 and !00 at 0x%lx\n",aposn);
      }
      total_audio += size;
      ofs += size;
      p += 16;
      continue;
    }
    unknown_recs++;
    p += 16;
    if (type == 0x01) continue;
    if (type == 0x02) continue;
    ofs+= size;
  }

  if (verbose)
    printf("recs: %d unk: %d vid: 0x%x aud: 0x%x total: 0x%x ofs=0x%x\n",
	   num_recs, unknown_recs, total_video, total_audio,
	   total_audio + total_video + num_recs * 16 + 4, ofs);
}

void usage(void)
{
  printf("vsplit [options] <infile> <audiofile> <videofile>\n");
  printf("options:\n");
  printf("    -v  verbose\n");
  printf("    -h  help\n");
  exit(1);
}

int main(int argc, char *argv[])
{
  char *fname;
  FILE *zin;
  unsigned char buf[CHUNK_SIZE];
  extern char *optarg;
  extern int optind;
  int c;

  while ((c = getopt(argc, argv, "vh")) != -1) {
    switch (c) {
    case 'v': verbose++; break;
    case 'h':
    default: usage(); exit(1);
    }
  }

  argc -= optind; argv += optind;

  if (argc < 3) { usage(); }

  fname = argv[0];
  zin = fopen(fname, "r");
  if (zin == NULL) { printf("Cannot read file %s\n", fname); exit(1); }
  audio_file = fopen(argv[1], "w");
  if (audio_file== NULL) { printf("Cannot write file %s\n", argv[2]); exit(1); }
  video_file = fopen(argv[2], "w");
  if (video_file== NULL) { printf("Cannot write file %s\n", argv[1]); exit(1); }

  /* Read the first chunk. If it doesn't contain F5 46 XX BD, then it's
   * a valid chunk, else skip it.
   */
  fread(buf, 1, CHUNK_SIZE, zin);
  if ((buf[0]!=0xf5) && (buf[1]!=0x46) && (buf[3]!=0xbd)) parse_chunk(buf);
  else printf("Skipped useless chunk 0\n");

  while (fread(buf, 1, CHUNK_SIZE, zin) == CHUNK_SIZE) parse_chunk(buf);

  fclose(audio_file);
  fclose(video_file);
  exit(1);
}
